# Generic Makefile to run tests of one or more programs
# Last edited on 2013-10-31 20:29:53 by stolfilocal

######################################################################
# To be included by the top-level Makefile in the program's directory.
# The caller must define
#
#   ${PROGDIR}           dir containing the executable program to be tested and its sources
#   ${PROG} or ${PROGS}  names of test programs (WITHOUT dir or extension)
#   
# The client must define the "all", "run", and "clean" targets.

SHELL := /bin/bash

#---------------------------------------------------------------------
# Phony "make" targets:

.PHONY:: \
  default-action \
  all actions debug \
  depend \
  build \
  depend-lib depend-libs build-lib build-libs do-build-lib \
  depend-prog depend-progs build-prog build-progs do-build-progs \
  depend-tests build-tests do-build-tests check \
  clean clean-lib clean-libs clean-prog clean-progs clean-tests

# ----------------------------------------------------------------------
# This will usually be the first action encountered by "make". 
all: check

ifeq "/${PROGS}" "/"
PROGS := ${PROG}
endif

ifneq "/${PROGS}" "/"
ifneq "/${PROGDIR}" "/"

#---------------------------------------------------------------------
# "make build" and "make build-progs" make sure that the
# executables in ${PROGDIR} are up to date:

build: build-progs
  
PROG_XFILES := ${addprefix ${PROGDIR}/, ${PROGS}}

build-progs: ${PROG_XFILES}

${PROG_XFILES}: \
          ${wildcard ${PROGDIR}/*.h ${PROGDIR}/*.c}
	@echo "rebuilding ${PROG_XFILES} ..."
	cd ${PROGDIR}/. && ${MAKE} ${PROGS}

#---------------------------------------------------------------------
# "make clean" deletes all derived files.
# It is a double-colon rule so that clients may add more actions.

clean::
        
endif
endif
# End of ${PROG} ${PROGDIR} section.
######################################################################

#---------------------------------------------------------------------
# "make debug" prints some debugging info:
debug:
	@echo "PROG = ${PROG}"
	@echo "PROGS = ${PROGS}"
	@echo "PROG_DIR = ${PROG_DIR}"
